package Gtk2::Builder::Simple;

use 5.008;
use strict;
use warnings;
use Carp;
use Gtk2;

our $VERSION = '0.03';

sub new {
    my( $caller, $builder_data, $domain ) = @_;
    my $self = bless {}, ref( $caller ) || $caller;

    croak( 'You need to specify a builder file or string first' ),
		unless( defined( $builder_data ) );

    $self->{builder} = Gtk2::Builder->new;

	$self->builder_object->set_translation_domain( $domain ),
		if( defined( $domain ) );

	eval {
		if( $builder_data =~ m{<.*?>}s ) {
			$self->builder_object->add_from_string( $builder_data );
		} else {
			$self->builder_object->add_from_file( $builder_data );
		}
	};
	if( $@ ) {
		return undef;
	}

    $self->__signal_autoconnect_simple;
    $self->__get_widgets;

    return $self;
}

sub builder_object {
    my( $self ) = shift;

    return $self->{builder};
}

sub get_object {
    my( $self, $object ) = @_;

    return $self->builder_object->get_object( $object );
}

sub get_objects {
    my( $self ) = shift;

    return $self->builder_object->get_objects;
}

sub get_widget {
    my( $self, $widget ) = @_;

    return ( exists( $self->{$widget} ) ) ? $self->{$widget} : undef;
}

sub get_widgets {
    my( $self ) = shift;
	my @list = ();

	foreach my $object ( $self->get_objects ) {
		push( @list, $object ) if( $object->isa( 'Gtk2::Widget' ) );
	}

    return @list;
}

sub run {
    my( $self ) = shift;

    Gtk2->main;
}

sub __get_widgets {
    my( $self ) = shift;

	foreach my $object ( $self->get_widgets ) {
		$self->{$object->get_name} = $object;
	}
}

my $_autoconnect_helper = sub {
    my( $builder,
		$object,
		$signal_name,
		$handler_name,
		$connect_object,
		$flags,
		$self
	) = @_;
    my $connect_func = (
		( $flags & 'after' ) ?
			'signal_connect_after' : 'signal_connect'
	);

    $object->$connect_func(
		$signal_name => sub {
			$self->$handler_name( @_ )
		},
		$connect_object
	);
};

sub __signal_autoconnect_simple {
    my( $self ) = shift;

    $self->builder_object->connect_signals_full( $_autoconnect_helper, $self );
}


1;
__END__

=head1 NAME

Gtk2::Builder::Simple - A clean object-oriented interface to Gtk2::Builder

=head1 SYNOPSIS

   package MyApp;
   use base qw( Gtk2::Builder::Simple );

   sub new {
      my $class = shift;
      my $self = $class->SUPER::new( $builder_data );
      return $self;
   }

   ...

   # Signal handlers are methods of your class
   sub on_button_clicked {
      my $self = shift;
      # You have access to your widgets directly
      # or using $self->get_widget( widget_name )
      my $button = $self->{button1};
   }

=head1 DESCRIPTION

Gtk2::Builder::Simple is a module that provides a clean and easy interface
for Gnome/Gtk2 and GtkBuilder applications using an object-oriented syntax. You just
make Gtk2::Builder::Simple your application's base class, have your C<new> call
C<SUPER::new>, and the module will do the tedious and dirty work for you.

It is based on Gtk2::GladeXML::Simple by Marco Antonio Manzo and tries to be a drop
in replacement for switching from Glade to GtkBuilder.

Gtk2::Builder::Simple offers:

=over

=item *

Signal handler callbacks as methods of your class.

   sub on_button1_clicked {
      my $self = shift; # $self always received as first parameter
      ...
      # do anything you want in a OO fashioned way
   }

=item *

Autoconnection of signal handlers.

=item *

Access to the widgets as instance attributes.

   my $btn = $self->{button1}; # fetch widgets as instance attributes by their names
   my $window = $self->{main_window};
   my $custom = $self->{custom_widget};

=back


=head1 METHODS

This class provides the following public methods:

=over

=item new( $builder_data I<[, $domain ]> );

This method creates a new object of your subclass of Gtk2::Builder::Simple.
The C<$builder_data> parameter is the name of the file created by the Glade Visual Editor
(saved in GtkBuilder format), or a string containing the GtkBuilder XML data.
The optional C<$domain> parameter that specifies the translation domain for the GtkBuilder
XML file ( undef by default ).

=item builder_object

This method returns the Gtk2::Builder object in play.

=item get_object( $object_name )

Returns the object with given name. Useful for non-widget type objects.

=item get_objects

Returns a list with all the objects in the GtkBuilder file.

=item get_widget( $widget_name )

Returns the widget with given name. Same as calling $self->{$widget_name}.

=item get_widgets

Returns a list with all the widgets in the GtkBuilder file as objects.

=item run

Call this method in order to run your application. If you need another event loop
rather than the Gtk one, override I<run> in your class with your event loop (for
example the GStreamer event loop).

=back

=head1 EXTENDED EXAMPLE

This example shows the usage of the module by creating a small file viewer.

   package Viewer;

   use strict;
   use warnings;

   use Gtk2 qw( -init );
   use Gtk2::Builder::Simple;
   use IO::File;

   use base qw( Gtk2::Builder::Simple );

   sub new {
       my $class = shift;
       my $self = $class->SUPER::new( 'viewer.ui' );
       return $self;
   }

   sub on_open_activate {
       my $self = shift;
       my $dlg = Gtk2::FileChooserDialog->new(
           'Please choose file to open',
           $self->{window},
           'open',
           _Cancel => 'cancel',
           _Open => 'ok',
       );

       if( 'ok' eq $dlg->run ) {
           # Hide dialog in case there is an error
           $dlg->hide;
           $self->load_file( $dlg->get_filename );
       }

       $dlg->destroy;
   }

   sub load_file {
       my $self = shift;
       my ($file) = @_;
       my $buffer = $self->{textview}->get_buffer;

       $buffer->set_text( '' );

       eval {
           my $fh = IO::File->new;
           my $iter = $buffer->get_start_iter;

           $fh->open( $file, '<:utf8' ) or die( $! );

           undef( $! );
           for( my $line;; ) {
               unless( defined( $line = $fh->getline ) ) {
                   if( $! ) {
                       $buffer->set_text( '' );
                       die( $! );
                   }

                   last;
               }

               $buffer->insert( $iter, $line );
           }

           $fh->close;
       };
       if( $@ ) {
           my $dialog = Gtk2::MessageDialog->new(
               $self->{window},
               [ qw( modal destroy-with-parent ) ],
               'error',
               'ok',
               'Failed to read the file.'
           );

           $dialog->run;
           $dialog->destroy;
       }
   }

   sub gtk_main_quit {
       Gtk2->main_quit;
   }


   1;

   package main;

   Viewer->new->run;


   1;

The I<viewer.ui> file needed for this example is in the I<examples> directory,
along with other example programs.

=head1 UTILITIES

=head2 Rapid Application Development with I<gbpsketcher>

The Gtk2::Builder::Simple distribution includes I<gbpsketcher>, a program that
generates Perl code stubs from GtkBuilder XML files. The code stubs include the basic
framework for Gtk2::Builder::Simple interaction, method signatures, and everything
that describes the application itself. Developers must fill in the code stubs to
add the correct functionality to the application.

=head1 SEE ALSO

L<Gtk2::Builder>, L<Gtk2>, L<gbpsketcher>

The Interface Builder Reference Manual at L<http://library.gnome.org/devel/gtk/stable/Builder.html>

The gtk2 API Reference at L<http://library.gnome.org/devel/gtk/stable/index.html>

=head1 TODO

Tests.

More examples?

Support to I18N ( bindtextdomain )

=head1 AUTHOR

Martin Schlemmer <11285613@nwu.ac.za>

Special thanks to Marco Antonio Manzo for the original Gtk2::GladeXML::Simple.

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Marco Antonio Manzo

Copyright (C) 2009 by CTexT, North-West University

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.6 or,
at your option, any later version of Perl 5 you may have available.

=cut
